<?php

namespace Drupal\content_reporting\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Database\Query\PagerSelectExtender;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\StreamedResponse;

class ContentReportingController extends ControllerBase {

  protected $currentUser;

  public function __construct(AccountProxyInterface $current_user) {
    $this->currentUser = $current_user;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_user')
    );
  }
  public function getReport(Request $request) {
    // Check user permissions
    if (!$this->currentUser->hasPermission('content reporting view')) {
      return new JsonResponse([
        'status' => 'error',
        'message' => 'Access denied',
      ], 403);  // Return 403 if the user does not have permission
    }

    // Get filters from request
    $filters = [
      'title' => $request->query->get('title'),
      'start_date' => $request->query->get('start_date'),
      'end_date' => $request->query->get('end_date'),
      'gdpr_consent' => $request->query->get('gdpr_consent'),
    ];

    // Define table headers with sorting attributes
    $header = [
      ['data' => t('Report ID'), 'field' => 'r.id'],
      ['data' => t('Page Title'), 'field' => 'r.title'],
      ['data' => t('Views'), 'field' => 'r.views'],
      ['data' => t('Clicks')],  // Field not directly in the DB
      ['data' => t('Time Spent')],  // Calculated field
      ['data' => t('GDPR count'), 'field' => 'r.gdpr_consent'],
    ];

    // Main query to get report data
    $query = \Drupal::database()->select('content_reporting_reports', 'r')
      ->fields('r', ['id', 'title', 'views', 'gdpr_consent', 'nid'])
      ->extend(PagerSelectExtender::class)
      ->limit(10);
    // Definisci i filtri e le opzioni di ordinamento
    $sort = $request->query->get('order') ? $request->query->get('order') : 'id';
    $order = $request->query->get('sort') ? strtoupper($request->query->get('sort')) : 'ASC';
    if (!$order) {
      $order = 'ASC';
    }
    // Applica l'ordinamento alla query
    if (!empty($sort) && in_array($sort, ['id', 'title', 'views', 'clicks', 'time_spent', 'gdpr_consent'])) {
      $query->orderBy($sort, $order);
    } else {
      // Default sorting per ID
      $query->orderBy('id', 'ASC');
    }
    // Apply filters to the main query
    if (!empty($filters['title'])) {
      $query->condition('title', '%' . $filters['title'] . '%', 'LIKE');
    }
    if (!empty($filters['start_date'])) {
      $query->condition('report_date', strtotime($filters['start_date']), '>=');
    }
    if (!empty($filters['end_date'])) {
      $query->condition('report_date', strtotime($filters['end_date']), '<=');
    }


    // Filtri basati sugli intervalli percentuali selezionati
    if (!empty($filters['gdpr_consent'])) {
      // Aggiungi una espressione alla query per calcolare la percentuale dei consensi GDPR
      $query->addExpression('(r.gdpr_consent / r.views) * 100', 'gdpr_percentage');
      switch ($filters['gdpr_consent']) {
        case '0_cons':
          // Nessun consenso (0 consensi)
          $query->condition('r.gdpr_consent', 0);
          break;

        case '0_25':
          // Dal 0% al 25% di consensi
          $query->having('gdpr_percentage >= :min', [':min' => 0])
            ->having('gdpr_percentage <= :max', [':max' => 25]);
          break;

        case '25_50':
          // Dal 25% al 50% di consensi
          $query->having('gdpr_percentage >= :min', [':min' => 25])
            ->having('gdpr_percentage <= :max', [':max' => 50]);
          break;

        case '50_70':
          // Dal 50% al 70% di consensi
          $query->having('gdpr_percentage >= :min', [':min' => 50])
            ->having('gdpr_percentage <= :max', [':max' => 70]);
          break;

        case '70_100':
          // Dal 70% al 100% di consensi
          $query->having('gdpr_percentage >= :min', [':min' => 70])
            ->having('gdpr_percentage <= :max', [':max' => 100]);
          break;
      }
    }
    // Execute the main query
    $results = $query->execute()->fetchAll();

    // Prepare rows for table
    $rows = [];
    foreach ($results as $result) {
      // Query per ottenere tutte le interazioni del tipo "click" per un determinato nid
      $clicks_query = \Drupal::database()->select('content_reporting_interactions', 'i')
        ->fields('i', ['nid', 'id', 'interaction_type', 'element'])
        ->condition('i.interaction_type', 'click')
        ->condition('i.nid', $result->nid);

      $clicks_result = $clicks_query->execute()->fetchAll();

      // Raccogli i click per ogni elemento e contali
      $clicks_data = [];
      foreach ($clicks_result as $click) {
        $element = strlen($click->element) > 5 ? substr($click->element, 0, 255) : $click->element;
        if (!isset($clicks_data[$element])) {
          $clicks_data[$element] = 0;
        }
        $clicks_data[$element]++;
      }

      // Genera una tabella HTML per i click
      $clicks_table = '<table><tr><th>Element</th><th>Count</th></tr>';
      foreach ($clicks_data as $element => $count) {
        $clicks_table .= '<tr><td>' . $element . '</td><td>' . $count . '</td></tr>';
      }
      $clicks_table .= '</table>';
      // Query per ottenere tutte le interazioni del tipo "time_spent" per un determinato nid
      $time_spent_query = \Drupal::database()->select('content_reporting_interactions', 'i')
        ->fields('i', ['nid', 'duration', 'interaction_type'])
        ->condition('i.interaction_type', 'time_spent')
        ->condition('i.nid', $result->nid);

      $time_spent_result = $time_spent_query->execute()->fetchAll();

      // Somma il "time_spent" in PHP
      $time_spent = 0;
      foreach ($time_spent_result as $record) {
        $time_spent += $record->duration;
      }
      // Registra i risultati, non la query
      \Drupal::logger('content_reporting')->info('Clicks for nid @nid: @clicks Time spent @time_spent', [
        '@nid' => $result->id,
        '@clicks' => Markup::create($clicks_table),
        '@time_spent' => $time_spent,
      ]);
      // Handle cases where there might be no data
      $rows[] = [
        'data' => [
          $result->id,
          $result->title,
          $result->views,
          $clicks_result !== FALSE && $clicks_data ?
            [
              'data' => Markup::create($clicks_table),  // Usa Markup per il rendering HTML
              'class' => ['clicks-column'],  // Classe CSS opzionale
            ]
            : 0,
          $time_spent !== FALSE ? $time_spent : 0,
          $result->gdpr_consent,
        ],
      ];
    }

    $build['#attached']['library'][] = 'content_reporting/content_reporting.styles';

    // Add form elements for filters
    $build['filters'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['filters-container']],
    ];

    // Optional: Add form and form API to manage filters
    $build['filters']['filters_form'] = \Drupal::formBuilder()->getForm('Drupal\content_reporting\Form\ContentReportingFiltersForm');
    $build['filters']['export_link'] = [
      '#type' => 'link',
      '#title' => t('Export to CSV'),
      '#url' => Url::fromRoute('content_reporting.export_to_csv', [], ['query' => \Drupal::request()->query->all()]),
      '#attributes' => [
        'class' => ['button'],
      ],
    ];
    // Build table
    $build['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => t('No data available.'),
    ];

    // Add pager
    $build['pager'] = ['#type' => 'pager'];


    return $build;
  }

  // Aggiungi un nuovo metodo per l'esportazione in CSV
  public function exportToCsv(Request $request) {
    // Verifica i permessi
    if (!$this->currentUser->hasPermission('content reporting view')) {
      return new JsonResponse([
        'status' => 'error',
        'message' => 'Access denied',
      ], 403);
    }

    $response = new StreamedResponse(function () use ($request) {
      // Apri il file CSV
      $handle = fopen('php://output', 'w+');

      // Aggiungi l'intestazione del file CSV
      fputcsv($handle, [
        'Report ID',
        'Page Title',
        'Views',
        'Clicks',
        'Time Spent',
        'GDPR Consent'
      ]);

      // Esegui la query per ottenere i dati
      $query = \Drupal::database()->select('content_reporting_reports', 'r')
        ->fields('r', ['id', 'title', 'views', 'gdpr_consent', 'nid']);

      // Applica i filtri (opzionale, puoi usare quelli del metodo `getReport`)
      $results = $query->execute()->fetchAll();

      // Prepara i dati per il CSV
      foreach ($results as $result) {
        // Ottieni i click e il tempo trascorso come nel metodo `getReport`
        $clicks_query = \Drupal::database()->select('content_reporting_interactions', 'i')
          ->fields('i', ['nid', 'id', 'interaction_type', 'element'])
          ->condition('i.interaction_type', 'click')
          ->condition('i.nid', $result->nid);
        $clicks_result = $clicks_query->execute()->fetchAll();

        $clicks_data = [];
        foreach ($clicks_result as $click) {
          $element = strlen($click->element) > 5 ? substr($click->element, 0, 5) : $click->element;
          if (!isset($clicks_data[$element])) {
            $clicks_data[$element] = 0;
          }
          $clicks_data[$element]++;
        }

        $clicks_cells = [];
        foreach ($clicks_data as $element => $count) {
          $clicks_cells[] = "$element: $count";
        }

        $time_spent_query = \Drupal::database()->select('content_reporting_interactions', 'i')
          ->fields('i', ['nid', 'duration', 'interaction_type'])
          ->condition('i.interaction_type', 'time_spent')
          ->condition('i.nid', $result->nid);
        $time_spent_result = $time_spent_query->execute()->fetchAll();

        $time_spent = 0;
        foreach ($time_spent_result as $record) {
          $time_spent += $record->duration;
        }

        // Scrivi la riga nel file CSV
        fputcsv($handle, [
          $result->id,
          $result->title,
          $result->views,
          implode(", ", $clicks_cells),
          $time_spent,
          $result->gdpr_consent,
        ]);
      }

      // Chiudi il file CSV
      fclose($handle);
    });

    // Configura le intestazioni per il download del CSV
    $response->setStatusCode(200);
    $response->headers->set('Content-Type', 'text/csv; charset=utf-8');
    $response->headers->set('Content-Disposition', 'attachment; filename="content-report.csv"');

    return $response;
  }
}
