<?php

namespace Drupal\content_reporting\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Drupal\node\Entity\Node;
use Drupal\Core\Database\Connection;
use Drupal\Core\Session\AccountProxyInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class TrackingController extends ControllerBase {

  protected $database;
  protected $currentUser;

  public function __construct(Connection $database, AccountProxyInterface $current_user) {
    $this->database = $database;
    $this->currentUser = $current_user;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('current_user')
    );
  }

  public function trackContent(Request $request) {
    // Verifica il token CSRF
    $csrf_token = $request->headers->get('X-CSRF-Token');
    if (!$csrf_token || !$this->isValidCSRFToken($csrf_token)) {
      throw new AccessDeniedHttpException('Invalid CSRF token.');
    }

    // Recupera il body della richiesta
    $data = json_decode($request->getContent(), TRUE);
    if (isset($data['nid'])) {
      $node_id = intval($data['nid']);
      $node = Node::load($node_id);

      if ($node) {
        $node_title = $node->getTitle();
        $gdpr_consent = isset($data['gdpr_consent']) && $data['gdpr_consent'] == true ? $data['gdpr_consent'] : NULL;

        $this->trackNode($node_id, $node_title, $gdpr_consent);

        return new JsonResponse(['status' => 'success']);
      }
      return new JsonResponse(['status' => 'error', 'message' => 'Node not found'], Response::HTTP_BAD_REQUEST);
    }

    return new JsonResponse(['status' => 'error', 'message' => 'Missing node ID'], Response::HTTP_BAD_REQUEST);
  }

  protected function trackNode($node_id, $node_title, $gdpr_consent) {
    $uid = $this->currentUser->isAuthenticated() ? $this->currentUser->id() : 0;

    // Controllo più rigoroso del consenso GDPR
    $consent = is_null($gdpr_consent) ? NULL : (int) $gdpr_consent;

    // Inserisci o aggiorna il record nel database
    $query = $this->database->merge('content_reporting_reports')
      ->key(['nid' => $node_id, 'uid' => $uid])
      ->fields([
        'title' => $node_title,
        'views' => 1,
        'report_date' => REQUEST_TIME,
      ])
      ->expression('views', 'COALESCE(views, 0) + 1');
      // Aggiorna il conteggio dei consensi se il consenso è stato dato (gdpr_consent == 1)
      if ($consent == true) {
        $query->expression('gdpr_consent', 'COALESCE(gdpr_consent, 0) + 1');
      }
    // Esegui la query per il record principale
    $query->execute();
  }

  public function trackInteraction(Request $request) {
    // Verifica il token CSRF
    $csrf_token = $request->headers->get('X-CSRF-Token');
    if (!$csrf_token || !$this->isValidCSRFToken($csrf_token)) {
      throw new AccessDeniedHttpException('Invalid CSRF token.');
    }
  
    // Recupera il body della richiesta
    $data = json_decode($request->getContent(), TRUE);
  
    if (isset($data['nid']) && isset($data['interaction_type'])) {
      $node_id = intval($data['nid']);
      $uid = $this->currentUser->isAuthenticated() ? $this->currentUser->id() : 0;
      $element = isset($data['element']) ? $data['element'] : NULL;
      $duration = isset($data['duration']) ? $data['duration'] : NULL;
      $interaction_type = $data['interaction_type'];
      
      // Salva l'interazione nel database
      \Drupal::database()->insert('content_reporting_interactions')
        ->fields([
          'nid' => $node_id,
          'uid' => $uid,
          'interaction_type' => $interaction_type,
          'element' => $element,
          'duration' => $duration,
          'timestamp' => REQUEST_TIME,
        ])
        ->execute();
  
      return new JsonResponse(['status' => 'success']);
    }
  
    return new JsonResponse(['status' => 'error', 'message' => 'Missing data'], Response::HTTP_BAD_REQUEST);
  }  
  
  protected function isValidCSRFToken($token) {
    // Verifica la validità del token CSRF (questa è solo un'implementazione fittizia)
    return TRUE;
    // In un contesto reale, puoi usare il servizio csrf_token per verificare la validità:
    // return \Drupal::csrfToken()->validate($token);
  }
}
